@@ -436,6 +436,7 @@ update_timezone (CcDateTimePanel *self)
   widget = (GtkWidget *) gtk_builder_get_object (priv->builder,
                                                  "city_combobox");
   model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
+  gtk_tree_model_filter_refilter ((GtkTreeModelFilter *) gtk_builder_get_object (priv->builder, "city-modelfilter"));
   gtk_tree_model_get_iter_first (model, &iter);
 
   do
@@ -512,6 +513,7 @@ get_regions (TzLocation             *loc,
   gchar *zone;
   gchar **split;
   gchar **split_translated;
+  gchar *translated_city;
 
   zone = g_strdup (loc->zone);
   g_strdelimit (zone, "_", ' ');
@@ -533,15 +535,19 @@ get_regions (TzLocation             *loc,
                                          REGION_COL_REGION_TRANSLATED, split_translated[0], -1);
     }
 
+  /* g_regex_split_simple() splits too much for us, and would break
+   * America/Argentina/Buenos_Aires into 3 strings, so rejoin the city part */
+  translated_city = g_strjoinv ("/", split_translated + 1);
 
   gtk_list_store_insert_with_values (data->city_store, NULL, 0,
                                      CITY_COL_CITY, split[1],
-                                     CITY_COL_CITY_TRANSLATED, split_translated[1],
+                                     CITY_COL_CITY_TRANSLATED, translated_city,
                                      CITY_COL_REGION, split[0],
                                      CITY_COL_REGION_TRANSLATED, split_translated[0],
                                      CITY_COL_ZONE, loc->zone,
                                      -1);
 
+  g_free (translated_city);
   g_strfreev (split);
   g_strfreev (split_translated);
 }
@@ -557,9 +563,10 @@ city_model_filter_func (GtkTreeModel *model,
   gchar *city_region = NULL;
   gboolean result;
 
+  if (gtk_combo_box_get_active_iter (combo, &combo_iter) == FALSE)
+    return FALSE;
 
   combo_model = gtk_combo_box_get_model (combo);
-  gtk_combo_box_get_active_iter (combo, &combo_iter);
   gtk_tree_model_get (combo_model, &combo_iter,
                       CITY_COL_CITY, &active_region, -1);
 
@@ -837,8 +844,6 @@ cc_date_time_panel_init (CcDateTimePanel *self)
 
   update_time (self);
 
-  get_system_timezone_async ((GetTimezoneFunc) get_timezone_cb, self, NULL);
-
   priv->locations = (GtkTreeModel*) gtk_builder_get_object (priv->builder,
                                                             "region-liststore");
 
@@ -859,10 +864,20 @@ cc_date_time_panel_init (CcDateTimePanel *self)
                                           widget,
                                           NULL);
 
+  /* After the initial setup, so we can be sure that
+   * the model is filled up */
+  get_system_timezone_async ((GetTimezoneFunc) get_timezone_cb, self, NULL);
+
   queue_clock_update (self);
 
   /* add the lock button */
   permission = polkit_permission_new_sync ("org.gnome.settingsdaemon.datetimemechanism.configure", NULL, NULL, NULL);
+  if (permission == NULL)
+    {
+      g_warning ("Your system does not have the '%s' PolicyKit files installed. Please check your installation",
+                 "org.gnome.settingsdaemon.datetimemechanism.configure");
+      return;
+    }
 
   /* DtLockButton takes ownership of the permission */
   lockbutton = dt_lock_button_new (permission);
