@@ -35,11 +35,20 @@
 #include <glib/gi18n.h>
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-bindings.h>
+#include <gsettings-desktop-schemas/gdesktop-enums.h>
 
 #include "xrandr-capplet.h"
 
 #define TOP_BAR_HEIGHT 10
 
+#define CLOCK_SCHEMA "org.gnome.desktop.interface"
+#define CLOCK_FORMAT_KEY "clock-format"
+
+/* The minimum supported size for the panel, see:
+ * http://live.gnome.org/Design/SystemSettings */
+#define MINIMUM_WIDTH 675
+#define MINIMUM_HEIGHT 530
+
 typedef struct App App;
 typedef struct GrabInfo GrabInfo;
 
@@ -50,6 +59,7 @@ struct App
   GnomeRRLabeler *labeler;
   GnomeRROutputInfo         *current_output;
 
+  GSettings      *clock_settings;
   GtkBuilder     *builder;
 
   GtkWidget      *panel;
@@ -64,7 +74,6 @@ struct App
   GtkWidget      *show_icon_checkbox;
 
   /* We store the event timestamp when the Apply button is clicked */
-  GtkWidget      *apply_button;
   guint32         apply_button_clicked_timestamp;
 
   GtkWidget      *area;
@@ -75,11 +84,6 @@ struct App
   DBusGConnection *connection;
   DBusGProxy *proxy;
   DBusGProxyCall *proxy_call;
-
-  enum {
-    APPLYING_VERSION_1,
-    APPLYING_VERSION_2
-  } apply_configuration_state;
 };
 
 static void rebuild_gui (App *app);
@@ -131,6 +135,20 @@ idle_free (gchar *s)
   return s;
 }
 
+static gboolean
+should_show_resolution (gint output_width,
+                        gint output_height,
+                        gint width,
+                        gint height)
+{
+  if (width >= MIN (output_width, MINIMUM_WIDTH) &&
+      height >= MIN (output_height, MINIMUM_HEIGHT))
+    {
+      return TRUE;
+    }
+  return FALSE;
+}
+
 static void
 on_screen_changed (GnomeRRScreen *scr,
                    gpointer data)
@@ -313,9 +331,9 @@ rebuild_rotation_combo (App *app)
   } RotationInfo;
   static const RotationInfo rotations[] = {
     { GNOME_RR_ROTATION_0, N_("Normal") },
-    { GNOME_RR_ROTATION_90, N_("Left") },
-    { GNOME_RR_ROTATION_270, N_("Right") },
-    { GNOME_RR_ROTATION_180, N_("Upside Down") },
+    { GNOME_RR_ROTATION_90, N_("Anti-Clockwise") },
+    { GNOME_RR_ROTATION_270, N_("Clockwise") },
+    { GNOME_RR_ROTATION_180, N_("180 Degrees") },
   };
   const char *selection;
   GnomeRRRotation current;
@@ -485,23 +503,31 @@ rebuild_current_monitor_label (App *app)
 
   if (use_color)
     {
-      GdkColor black = { 0, 0, 0, 0 };
+      GdkRGBA black = { 0, 0, 0, 1.0 };
+      GdkRGBA light;
 
-      gtk_widget_modify_bg (app->current_monitor_event_box, gtk_widget_get_state (app->current_monitor_event_box), &color);
+      light.red = color.red / 65535.0;
+      light.green = color.green / 65535.0;
+      light.blue = color.blue / 65535.0;
+      light.alpha = 1.0;
+      gtk_widget_override_background_color (app->current_monitor_event_box,
+                                            gtk_widget_get_state_flags (app->current_monitor_event_box),
+                                            &light);
 
       /* Make the label explicitly black.  We don't want it to follow the
        * theme's colors, since the label is always shown against a light
        * pastel background.  See bgo#556050
        */
-      gtk_widget_modify_fg (app->current_monitor_label, gtk_widget_get_state (app->current_monitor_label), &black);
+      gtk_widget_override_color (app->current_monitor_label,
+                                 gtk_widget_get_state_flags (app->current_monitor_label),
+                                 &black);
     }
   else
     {
       /* Remove any modifications we did on the label's color */
-      GtkRcStyle *reset_rc_style;
-
-      reset_rc_style = gtk_rc_style_new ();
-      gtk_widget_modify_style (app->current_monitor_label, reset_rc_style); /* takes ownership of, and destroys, the rc style */
+      gtk_widget_override_color (app->current_monitor_label,
+                                 gtk_widget_get_state_flags (app->current_monitor_label),
+                                 NULL);
     }
 
     gtk_event_box_set_visible_window (GTK_EVENT_BOX (app->current_monitor_event_box), use_color);
@@ -512,13 +538,11 @@ rebuild_on_off_radios (App *app)
 {
   gboolean sensitive;
   gboolean on_active;
-  gboolean off_active;
 
   g_signal_handlers_block_by_func (app->monitor_switch, G_CALLBACK (monitor_switch_active_cb), app);
 
   sensitive = FALSE;
   on_active = FALSE;
-  off_active = FALSE;
 
   if (!gnome_rr_config_get_clone (app->current_configuration) && app->current_output)
     {
@@ -528,7 +552,6 @@ rebuild_on_off_radios (App *app)
         sensitive = FALSE;
 
       on_active = gnome_rr_output_info_is_active (app->current_output);
-      off_active = !on_active;
     }
 
   gtk_widget_set_sensitive (app->monitor_switch, sensitive);
@@ -637,9 +660,12 @@ rebuild_resolution_combo (App *app)
       width = gnome_rr_mode_get_width (modes[i]);
       height = gnome_rr_mode_get_height (modes[i]);
 
-      add_key (app->resolution_combo,
-               idle_free (make_resolution_string (width, height)),
-               width, height, 0, -1);
+      if (should_show_resolution (output_width, output_height, width, height))
+        {
+          add_key (app->resolution_combo,
+                   idle_free (make_resolution_string (width, height)),
+                   width, height, 0, -1);
+        }
     }
 
   current = idle_free (make_resolution_string (output_width, output_height));
@@ -1478,7 +1504,7 @@ set_cursor (GtkWidget *widget, GdkCursorType type)
     gdk_window_set_cursor (window, cursor);
 
   if (cursor)
-    gdk_cursor_unref (cursor);
+    g_object_unref (cursor);
 }
 
 static void
@@ -1726,17 +1752,21 @@ paint_background (FooScrollArea *area,
 {
   GdkRectangle viewport;
   GtkWidget *widget;
-  GtkStyle *widget_style;
+  GtkStyleContext *context;
+  GdkRGBA fg, bg;
 
   widget = GTK_WIDGET (area);
 
   foo_scroll_area_get_viewport (area, &viewport);
-  widget_style = gtk_widget_get_style (widget);
+  context = gtk_widget_get_style_context (widget);
+  gtk_style_context_get_color (context, GTK_STATE_FLAG_NORMAL, &fg);
+  gtk_style_context_get_background_color (context, GTK_STATE_FLAG_NORMAL, &bg);
 
-  cairo_set_source_rgb (cr,
-                        widget_style->mid[GTK_STATE_NORMAL].red / 65535.0,
-                        widget_style->mid[GTK_STATE_NORMAL].green / 65535.0,
-                        widget_style->mid[GTK_STATE_NORMAL].blue / 65535.0);
+  cairo_set_source_rgba (cr,
+                         (fg.red + bg.red) / 2,
+                         (fg.green + bg.green) / 2,
+                         (fg.blue + bg.blue) / 2,
+                         (fg.alpha + bg.alpha) / 2);
 
   cairo_rectangle (cr,
                    viewport.x, viewport.y,
@@ -1746,10 +1776,11 @@ paint_background (FooScrollArea *area,
 
   foo_scroll_area_add_input_from_fill (area, cr, on_canvas_event, NULL);
 
-  cairo_set_source_rgb (cr,
-                        widget_style->dark[GTK_STATE_NORMAL].red / 65535.0,
-                        widget_style->dark[GTK_STATE_NORMAL].green / 65535.0,
-                        widget_style->dark[GTK_STATE_NORMAL].blue / 65535.0);
+  cairo_set_source_rgba (cr,
+                         0.7 * bg.red,
+                         0.7 * bg.green,
+                         0.7 * bg.blue,
+                         0.7 * bg.alpha);
 
   cairo_stroke (cr);
 }
@@ -1842,19 +1873,16 @@ paint_output (App *app, cairo_t *cr, int i)
 
   if (output == app->current_output)
     {
-      GtkStyle *style;
-      GdkColor  color;
+      GtkStyleContext *context;
+      GdkRGBA color;
 
-      style = gtk_widget_get_style (app->area);
-      color = style->bg[GTK_STATE_SELECTED];
-      r = (float)color.red / 65535.0;
-      g = (float)color.green / 65535.0;
-      b = (float)color.blue / 65535.0;
+      context = gtk_widget_get_style_context (app->area);
+      gtk_style_context_get_background_color (context, GTK_STATE_FLAG_SELECTED, &color);
 
       cairo_rectangle (cr, x - 2, y - 2, w * scale + 0.5 + 4, h * scale + 0.5 + 4);
 
       cairo_set_line_width (cr, 4);
-      cairo_set_source_rgba (cr, r, g, b, 0.5);
+      cairo_set_source_rgba (cr, color.red, color.green, color.blue, 0.5);
       cairo_stroke (cr);
     }
 
@@ -1918,6 +1946,7 @@ paint_output (App *app, cairo_t *cr, int i)
       char *text;
       gboolean use_24;
       GDateTime *dt;
+      GDesktopClockFormat value;
 
       /* top bar */
       cairo_rectangle (cr, x, y, w * scale + 0.5, TOP_BAR_HEIGHT);
@@ -1931,6 +1960,8 @@ paint_output (App *app, cairo_t *cr, int i)
 
       /* clock */
       /* FIXME: set 12/24 hour */
+      value = g_settings_get_enum (app->clock_settings, CLOCK_FORMAT_KEY);
+      use_24 = value == G_DESKTOP_CLOCK_FORMAT_24H;
       if (use_24)
         clock_format = _("%a %R");
       else
@@ -1973,7 +2004,6 @@ on_area_paint (FooScrollArea  *area,
                gpointer        data)
 {
   App *app = data;
-  double scale;
   GList *connected_outputs = NULL;
   GList *list;
 
@@ -1982,13 +2012,8 @@ on_area_paint (FooScrollArea  *area,
   if (!app->current_configuration)
     return;
 
-  scale = compute_scale (app);
   connected_outputs = list_connected_outputs (app, NULL, NULL);
 
-#if 0
-  g_debug ("scale: %f", scale);
-#endif
-
   for (list = connected_outputs; list != NULL; list = list->next)
     {
       paint_output (app, cr, g_list_position (connected_outputs, list));
@@ -2098,7 +2123,6 @@ begin_version2_apply_configuration (App *app, GdkWindow *parent_window, guint32
                                           "org.gnome.SettingsDaemon.XRANDR_2");
   g_assert (app->proxy != NULL); /* that call does not fail unless we pass bogus names */
 
-  app->apply_configuration_state = APPLYING_VERSION_2;
   app->proxy_call = dbus_g_proxy_begin_call (app->proxy, "ApplyConfiguration",
                                              apply_configuration_returned_cb, app,
                                              NULL,
@@ -2112,27 +2136,6 @@ begin_version2_apply_configuration (App *app, GdkWindow *parent_window, guint32
    */
 }
 
-static void
-begin_version1_apply_configuration (App *app)
-{
-  app->proxy = dbus_g_proxy_new_for_name (app->connection,
-                                          "org.gnome.SettingsDaemon",
-                                          "/org/gnome/SettingsDaemon/XRANDR",
-                                          "org.gnome.SettingsDaemon.XRANDR");
-  g_assert (app->proxy != NULL); /* that call does not fail unless we pass bogus names */
-
-  app->apply_configuration_state = APPLYING_VERSION_1;
-  app->proxy_call = dbus_g_proxy_begin_call (app->proxy, "ApplyConfiguration",
-                                             apply_configuration_returned_cb, app,
-                                             NULL,
-                                             G_TYPE_INVALID,
-                                             G_TYPE_INVALID);
-  /* FIXME: we don't check for app->proxy_call == NULL, which could happen if
-   * the connection was disconnected.  This is left as an exercise for the
-   * reader.
-   */
-}
-
 static void
 ensure_current_configuration_is_saved (void)
 {
@@ -2175,21 +2178,10 @@ apply_configuration_returned_cb (DBusGProxy       *proxy,
   success = dbus_g_proxy_end_call (proxy, call_id, &error, G_TYPE_INVALID);
 
   if (!success) {
-    if (app->apply_configuration_state == APPLYING_VERSION_2
-        && g_error_matches (error, DBUS_GERROR, DBUS_GERROR_UNKNOWN_METHOD)) {
-      g_error_free (error);
-
-      g_object_unref (app->proxy);
-      app->proxy = NULL;
-
-      begin_version1_apply_configuration (app);
-      return;
-    } else {
-      /* We don't pop up an error message; gnome-settings-daemon already does that
-       * in case the selected RANDR configuration could not be applied.
-       */
-      g_error_free (error);
-    }
+    /* We don't pop up an error message; gnome-settings-daemon already does that
+     * in case the selected RANDR configuration could not be applied.
+     */
+    g_error_free (error);
   }
 
   g_object_unref (app->proxy);
@@ -2233,6 +2225,8 @@ apply (App *app)
   GError *error = NULL;
   GdkWindow *window;
 
+  app->apply_button_clicked_timestamp = gtk_get_current_event_time ();
+
   if (!sanitize_and_save_configuration (app))
     return;
 
@@ -2431,19 +2425,6 @@ dialog_map_event_cb (GtkWidget *widget, GdkEventAny *event, gpointer data)
 }
 
 
-static void
-apply_button_clicked_cb (GtkButton *button, gpointer data)
-{
-  App *app = data;
-
-  /* We simply store the timestamp at which the Apply button was clicked.
-   * We'll just wait for the dialog to return from gtk_dialog_run(), and
-   * *then* use the timestamp when applying the RANDR configuration.
-   */
-
-  app->apply_button_clicked_timestamp = gtk_get_current_event_time ();
-}
-
 static GtkWidget*
 _gtk_builder_get_widget (GtkBuilder *builder, const gchar *name)
 {
@@ -2455,6 +2436,10 @@ destroy_app (App *app)
 {
   g_object_unref (app->screen);
   g_object_unref (app->builder);
+
+  if (app->clock_settings != NULL)
+    g_object_unref (app->clock_settings);
+
   gnome_rr_labeler_hide (app->labeler);
   g_object_unref (app->labeler);
 
@@ -2497,6 +2482,8 @@ run_application (void)
       return NULL;
     }
 
+  app->clock_settings = g_settings_new (CLOCK_SCHEMA);
+
   app->panel = _gtk_builder_get_widget (builder, "display-panel");
 
   if (!app->panel)
@@ -2554,10 +2541,6 @@ run_application (void)
 
   gtk_container_add (GTK_CONTAINER (align), app->area);
 
-  app->apply_button = _gtk_builder_get_widget (builder, "apply_button");
-  g_signal_connect (app->apply_button, "clicked",
-                    G_CALLBACK (apply_button_clicked_cb), app);
-
   on_screen_changed (app->screen, app);
 
   g_signal_connect_swapped (_gtk_builder_get_widget (builder, "apply_button"),
