@@ -1,4 +1,5 @@
-/*
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*-
+ *
  * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -63,7 +64,7 @@ cc_media_panel_dispose (GObject *object)
     g_object_unref (self->priv->preferences);
     self->priv->preferences = NULL;
   }
-  
+
   G_OBJECT_CLASS (cc_media_panel_parent_class)->dispose (object);
 }
 
@@ -91,7 +92,7 @@ cc_media_panel_class_finalize (CcMediaPanelClass *klass)
 
 static char **
 remove_elem_from_str_array (char **v,
-			    const char *s)
+                            const char *s)
 {
   GPtrArray *array;
   guint idx;
@@ -115,7 +116,7 @@ remove_elem_from_str_array (char **v,
 
 static char **
 add_elem_to_str_array (char **v,
-		       const char *s)
+                       const char *s)
 {
   GPtrArray *array;
   guint idx;
@@ -136,12 +137,12 @@ add_elem_to_str_array (char **v,
 
 static int
 media_panel_g_strv_find (char **strv,
-			 const char *find_me)
+                         const char *find_me)
 {
   guint index;
 
   g_return_val_if_fail (find_me != NULL, -1);
-	
+
   for (index = 0; strv[index] != NULL; ++index) {
     if (g_strcmp0 (strv[index], find_me) == 0) {
       return index;
@@ -153,10 +154,10 @@ media_panel_g_strv_find (char **strv,
 
 static void
 autorun_get_preferences (CcMediaPanel *self,
-			 const char *x_content_type,
-			 gboolean *pref_start_app,
-			 gboolean *pref_ignore,
-			 gboolean *pref_open_folder)
+                         const char *x_content_type,
+                         gboolean *pref_start_app,
+                         gboolean *pref_ignore,
+                         gboolean *pref_open_folder)
 {
   char **x_content_start_app;
   char **x_content_ignore;
@@ -170,11 +171,11 @@ autorun_get_preferences (CcMediaPanel *self,
   *pref_ignore = FALSE;
   *pref_open_folder = FALSE;
   x_content_start_app = g_settings_get_strv (self->priv->preferences,
-					     PREF_MEDIA_AUTORUN_X_CONTENT_START_APP);
+                                             PREF_MEDIA_AUTORUN_X_CONTENT_START_APP);
   x_content_ignore = g_settings_get_strv (self->priv->preferences,
-					  PREF_MEDIA_AUTORUN_X_CONTENT_IGNORE);
+                                          PREF_MEDIA_AUTORUN_X_CONTENT_IGNORE);
   x_content_open_folder = g_settings_get_strv (self->priv->preferences,
-					       PREF_MEDIA_AUTORUN_X_CONTENT_OPEN_FOLDER);
+                                               PREF_MEDIA_AUTORUN_X_CONTENT_OPEN_FOLDER);
   if (x_content_start_app != NULL) {
     *pref_start_app = media_panel_g_strv_find (x_content_start_app, x_content_type) != -1;
   }
@@ -191,10 +192,10 @@ autorun_get_preferences (CcMediaPanel *self,
 
 static void
 autorun_set_preferences (CcMediaPanel *self,
-			 const char *x_content_type,
-			 gboolean pref_start_app,
-			 gboolean pref_ignore,
-			 gboolean pref_open_folder)
+                         const char *x_content_type,
+                         gboolean pref_start_app,
+                         gboolean pref_ignore,
+                         gboolean pref_open_folder)
 {
   char **x_content_start_app;
   char **x_content_ignore;
@@ -203,32 +204,32 @@ autorun_set_preferences (CcMediaPanel *self,
   g_assert (x_content_type != NULL);
 
   x_content_start_app = g_settings_get_strv (self->priv->preferences,
-					     PREF_MEDIA_AUTORUN_X_CONTENT_START_APP);
+                                             PREF_MEDIA_AUTORUN_X_CONTENT_START_APP);
   x_content_ignore = g_settings_get_strv (self->priv->preferences,
-					  PREF_MEDIA_AUTORUN_X_CONTENT_IGNORE);
+                                          PREF_MEDIA_AUTORUN_X_CONTENT_IGNORE);
   x_content_open_folder = g_settings_get_strv (self->priv->preferences,
-					       PREF_MEDIA_AUTORUN_X_CONTENT_OPEN_FOLDER);
+                                               PREF_MEDIA_AUTORUN_X_CONTENT_OPEN_FOLDER);
 
   x_content_start_app = remove_elem_from_str_array (x_content_start_app, x_content_type);
   if (pref_start_app) {
     x_content_start_app = add_elem_to_str_array (x_content_start_app, x_content_type);
   }
   g_settings_set_strv (self->priv->preferences,
-		       PREF_MEDIA_AUTORUN_X_CONTENT_START_APP, (const gchar * const*) x_content_start_app);
+                       PREF_MEDIA_AUTORUN_X_CONTENT_START_APP, (const gchar * const*) x_content_start_app);
 
   x_content_ignore = remove_elem_from_str_array (x_content_ignore, x_content_type);
   if (pref_ignore) {
     x_content_ignore = add_elem_to_str_array (x_content_ignore, x_content_type);
   }
   g_settings_set_strv (self->priv->preferences,
-		       PREF_MEDIA_AUTORUN_X_CONTENT_IGNORE, (const gchar * const*) x_content_ignore);
+                       PREF_MEDIA_AUTORUN_X_CONTENT_IGNORE, (const gchar * const*) x_content_ignore);
 
   x_content_open_folder = remove_elem_from_str_array (x_content_open_folder, x_content_type);
   if (pref_open_folder) {
     x_content_open_folder = add_elem_to_str_array (x_content_open_folder, x_content_type);
   }
   g_settings_set_strv (self->priv->preferences,
-		       PREF_MEDIA_AUTORUN_X_CONTENT_OPEN_FOLDER, (const gchar * const*) x_content_open_folder);
+                       PREF_MEDIA_AUTORUN_X_CONTENT_OPEN_FOLDER, (const gchar * const*) x_content_open_folder);
 
   g_strfreev (x_content_open_folder);
   g_strfreev (x_content_ignore);
@@ -240,7 +241,7 @@ static void
 update_media_sensitivity (CcMediaPanel *self)
 {
   gtk_widget_set_sensitive (GTK_WIDGET (gtk_builder_get_object (self->priv->builder, "media_handling_vbox")),
-			    ! g_settings_get_boolean (self->priv->preferences, PREF_MEDIA_AUTORUN_NEVER));
+                            ! g_settings_get_boolean (self->priv->preferences, PREF_MEDIA_AUTORUN_NEVER));
 }
 
 static void
@@ -255,19 +256,19 @@ custom_item_activated_cb (GtkAppChooserButton *button,
 
   if (g_strcmp0 (item, CUSTOM_ITEM_ASK) == 0) {
     autorun_set_preferences (self, content_type,
-			     FALSE, FALSE, FALSE);
+                             FALSE, FALSE, FALSE);
   } else if (g_strcmp0 (item, CUSTOM_ITEM_OPEN_FOLDER) == 0) {
     autorun_set_preferences (self, content_type,
-			     FALSE, FALSE, TRUE);
+                             FALSE, FALSE, TRUE);
   } else if (g_strcmp0 (item, CUSTOM_ITEM_DO_NOTHING) == 0) {
     autorun_set_preferences (self, content_type,
-			     FALSE, TRUE, FALSE);
+                             FALSE, TRUE, FALSE);
   }
 
   g_free (content_type);
 }
 
-static void 
+static void
 combo_box_changed_cb (GtkComboBox *combo_box,
                       gpointer user_data)
 {
@@ -291,10 +292,10 @@ combo_box_changed_cb (GtkComboBox *combo_box,
 
 static void
 prepare_combo_box (CcMediaPanel *self,
-		   GtkWidget *combo_box)
+                   GtkWidget *combo_box,
+                   const gchar *heading)
 {
   GtkAppChooserButton *app_chooser = GTK_APP_CHOOSER_BUTTON (combo_box);
-  GIcon *icon;
   gboolean pref_ask;
   gboolean pref_start_app;
   gboolean pref_ignore;
@@ -306,7 +307,7 @@ prepare_combo_box (CcMediaPanel *self,
 
   /* fetch preferences for this content type */
   autorun_get_preferences (self, content_type,
-			   &pref_start_app, &pref_ignore, &pref_open_folder);
+                           &pref_start_app, &pref_ignore, &pref_open_folder);
   pref_ask = !pref_start_app && !pref_ignore && !pref_open_folder;
 
   info = gtk_app_chooser_get_app_info (GTK_APP_CHOOSER (combo_box));
@@ -317,25 +318,20 @@ prepare_combo_box (CcMediaPanel *self,
     g_object_unref (info);
   }
 
-  icon = g_themed_icon_new (GTK_STOCK_DIALOG_QUESTION);
   gtk_app_chooser_button_append_custom_item (app_chooser, CUSTOM_ITEM_ASK,
                                              _("Ask what to do"),
-                                             icon);
-  g_object_unref (icon);
+                                             NULL);
 
-  icon = g_themed_icon_new (GTK_STOCK_CLOSE);
   gtk_app_chooser_button_append_custom_item (app_chooser, CUSTOM_ITEM_DO_NOTHING,
-                                             _("Do Nothing"),
-                                             icon);
-  g_object_unref (icon);
+                                             _("Do nothing"),
+                                             NULL);
 
-  icon = g_themed_icon_new ("folder-open");
   gtk_app_chooser_button_append_custom_item (app_chooser, CUSTOM_ITEM_OPEN_FOLDER,
-                                             _("Open Folder"),
-                                             icon);
-  g_object_unref (icon);
+                                             _("Open folder"),
+                                             NULL);
 
   gtk_app_chooser_button_set_show_dialog_item (app_chooser, TRUE);
+  gtk_app_chooser_button_set_heading (app_chooser, _(heading));
 
   if (pref_ask) {
     gtk_app_chooser_button_set_active_custom_item (app_chooser, CUSTOM_ITEM_ASK);
@@ -353,9 +349,9 @@ prepare_combo_box (CcMediaPanel *self,
   g_free (content_type);
 }
 
-static void 
+static void
 other_type_combo_box_changed (GtkComboBox *combo_box,
-			      CcMediaPanel *self)
+                              CcMediaPanel *self)
 {
   GtkTreeIter iter;
   GtkTreeModel *model;
@@ -373,19 +369,19 @@ other_type_combo_box_changed (GtkComboBox *combo_box,
     return;
   }
 
-  gtk_tree_model_get (model, &iter, 
-		      2, &x_content_type,
-		      -1);
+  gtk_tree_model_get (model, &iter,
+                      2, &x_content_type,
+                      -1);
 
   action_container = GTK_WIDGET (gtk_builder_get_object (self->priv->builder,
-							 "media_other_action_container"));
+                                                         "media_other_action_container"));
   if (self->priv->other_application_combo != NULL) {
     gtk_widget_destroy (self->priv->other_application_combo);
   }
 
   self->priv->other_application_combo = gtk_app_chooser_button_new (x_content_type);
   gtk_box_pack_start (GTK_BOX (action_container), self->priv->other_application_combo, TRUE, TRUE, 0);
-  prepare_combo_box (self, self->priv->other_application_combo);
+  prepare_combo_box (self, self->priv->other_application_combo, NULL);
   gtk_widget_show (self->priv->other_application_combo);
 
   g_free (x_content_type);
@@ -408,9 +404,11 @@ static void
 on_extra_options_button_clicked (GtkWidget    *button,
                                  CcMediaPanel *self)
 {
-  GtkWidget  *dialog;
+  GtkWidget *dialog;
+  GtkWidget *combo_box;
 
   dialog = GTK_WIDGET (gtk_builder_get_object (self->priv->builder, "extra_options_dialog"));
+  combo_box = GTK_WIDGET (gtk_builder_get_object (self->priv->builder, "media_other_type_combobox"));
   gtk_window_set_transient_for (GTK_WINDOW (dialog),
                                 GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (self))));
   gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
@@ -418,6 +416,8 @@ on_extra_options_button_clicked (GtkWidget    *button,
                     "response",
                     G_CALLBACK (on_extra_options_dialog_response),
                     self);
+  /* update other_application_combo */
+  other_type_combo_box_changed (GTK_COMBO_BOX (combo_box), self);
   gtk_window_present (GTK_WINDOW (dialog));
 }
 
@@ -436,28 +436,52 @@ media_panel_setup (CcMediaPanel *self)
   struct {
     const gchar *widget_name;
     const gchar *content_type;
+    const gchar *heading;
   } const defs[] = {
-    { "media_audio_cdda_combobox", "x-content/audio-cdda" },
-    { "media_video_dvd_combobox", "x-content/video-dvd" },
-    { "media_music_player_combobox", "x-content/audio-player" },
-    { "media_dcf_combobox", "x-content/image-dcf" },
-    { "media_software_combobox", "x-content/software" },
+    { "media_audio_cdda_combobox", "x-content/audio-cdda", N_("Select an application for audio CDs") },
+    { "media_video_dvd_combobox", "x-content/video-dvd", N_("Select an application for video DVDs") },
+    { "media_music_player_combobox", "x-content/audio-player", N_("Select an application to run when a music player is connected") },
+    { "media_dcf_combobox", "x-content/image-dcf", N_("Select an application to run when a camera is connected") },
+    { "media_software_combobox", "x-content/software", N_("Select an application for software CDs") },
+  };
+
+  struct {
+    const gchar *content_type;
+    const gchar *description;
+  } const other_defs[] = {
+    /* translators: these strings are duplicates of shared-mime-info
+     * strings, just here to fix capitalization of the English originals.
+     * If the shared-mime-info translation works for your language,
+     * simply leave these untranslated.
+     */
+    { "x-content/audio-dvd", N_("audio DVD") },
+    { "x-content/blank-bd", N_("blank Blu-ray disc") },
+    { "x-content/blank-cd", N_("blank CD disc") },
+    { "x-content/blank-dvd", N_("blank DVD disc") },
+    { "x-content/blank-hddvd", N_("blank HD DVD disc") },
+    { "x-content/video-bluray", N_("Blu-ray video disc") },
+    { "x-content/ebook-reader", N_("e-book reader") },
+    { "x-content/video-hddvd", N_("HD DVD video disc") },
+    { "x-content/image-picturecd", N_("Picture CD") },
+    { "x-content/video-svcd", N_("Super Video CD") },
+    { "x-content/video-vcd", N_("Video CD") }
   };
 
   for (n = 0; n < G_N_ELEMENTS (defs); n++) {
     prepare_combo_box (self,
-		       GTK_WIDGET (gtk_builder_get_object (builder, defs[n].widget_name)));
+                       GTK_WIDGET (gtk_builder_get_object (builder, defs[n].widget_name)),
+                       defs[n].heading);
   }
 
   other_type_combo_box = GTK_WIDGET (gtk_builder_get_object (builder, "media_other_type_combobox"));
 
   other_type_list_store = gtk_list_store_new (3,
-					      G_TYPE_ICON,
-					      G_TYPE_STRING,
-					      G_TYPE_STRING);
+                                              G_TYPE_ICON,
+                                              G_TYPE_STRING,
+                                              G_TYPE_STRING);
 
   gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (other_type_list_store),
-					1, GTK_SORT_ASCENDING);
+                                        1, GTK_SORT_ASCENDING);
 
 
   content_types = g_content_types_get_registered ();
@@ -472,19 +496,30 @@ media_panel_setup (CcMediaPanel *self)
 
     for (n = 0; n < G_N_ELEMENTS (defs); n++) {
       if (g_content_type_is_a (content_type, defs[n].content_type)) {
-	goto skip;
+        goto skip;
       }
     }
 
-    description = g_content_type_get_description (content_type);
+    for (n = 0; n < G_N_ELEMENTS (other_defs); n++) {
+       if (strcmp (content_type, other_defs[n].content_type) == 0) {
+         const gchar *s = other_defs[n].description;
+         if (s == _(s))
+           description = g_content_type_get_description (content_type);
+         else
+           description = g_strdup (_(s));
+
+         break;
+       }
+    }
+
     gtk_list_store_append (other_type_list_store, &iter);
     icon = g_content_type_get_icon (content_type);
 
     gtk_list_store_set (other_type_list_store, &iter,
-			0, icon,
-			1, description,
-			2, content_type,
-			-1);
+                        0, icon,
+                        1, description,
+                        2, content_type,
+                        -1);
     g_free (description);
     g_object_unref (icon);
   skip:
@@ -494,24 +529,24 @@ media_panel_setup (CcMediaPanel *self)
   g_list_free_full (content_types, g_free);
 
   gtk_combo_box_set_model (GTK_COMBO_BOX (other_type_combo_box),
-			   GTK_TREE_MODEL (other_type_list_store));
+                           GTK_TREE_MODEL (other_type_list_store));
 
   renderer = gtk_cell_renderer_pixbuf_new ();
   gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (other_type_combo_box), renderer, FALSE);
   gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (other_type_combo_box), renderer,
-				  "gicon", 0,
-				  NULL);
+                                  "gicon", 0,
+                                  NULL);
 
   renderer = gtk_cell_renderer_text_new ();
   gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (other_type_combo_box), renderer, TRUE);
   gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (other_type_combo_box), renderer,
-				  "text", 1,
-				  NULL);
+                                  "text", 1,
+                                  NULL);
 
   g_signal_connect (other_type_combo_box,
-		    "changed",
-		    G_CALLBACK (other_type_combo_box_changed),
-		    self);
+                    "changed",
+                    G_CALLBACK (other_type_combo_box_changed),
+                    self);
 
   gtk_combo_box_set_active (GTK_COMBO_BOX (other_type_combo_box), 0);
 
@@ -545,7 +580,7 @@ cc_media_panel_init (CcMediaPanel *self)
   media_panel_setup (self);
 
   main_widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
-						    "media_preferences_vbox"));
+                                                    "media_preferences_vbox"));
   gtk_widget_reparent (main_widget, (GtkWidget *) self);
 }
 
