@@ -24,10 +24,16 @@
 #include <string.h>
 #include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
+#include <polkit/polkit.h>
 #include <dbus/dbus-glib.h>
+#include <gsettings-desktop-schemas/gdesktop-enums.h>
 
 #include <cups/cups.h>
 
+#include <math.h>
+
+#include "pp-lockbutton.h"
+
 G_DEFINE_DYNAMIC_TYPE (CcPrintersPanel, cc_printers_panel, CC_TYPE_PANEL)
 
 #define PRINTERS_PANEL_PRIVATE(o) \
@@ -35,14 +41,19 @@ G_DEFINE_DYNAMIC_TYPE (CcPrintersPanel, cc_printers_panel, CC_TYPE_PANEL)
 
 #define MECHANISM_BUS "org.opensuse.CupsPkHelper.Mechanism"
 
-#define SUPPLY_BAR_WIDTH 140
-#define SUPPLY_BAR_HEIGHT 10
-#define SUPPLY_BAR_SPACE 5
+#define SUPPLY_BAR_HEIGHT 20
+
+#define EMPTY_TEXT "\xe2\x80\x94"
+
+#define CLOCK_SCHEMA "org.gnome.desktop.interface"
+#define CLOCK_FORMAT_KEY "clock-format"
 
 struct _CcPrintersPanelPrivate
 {
   GtkBuilder *builder;
 
+  GtkWidget *lock_button;
+
   cups_dest_t *dests;
   int num_dests;
   int current_dest;
@@ -57,6 +68,8 @@ struct _CcPrintersPanelPrivate
 
   GdkRGBA background_color;
 
+  GPermission *permission;
+
   gpointer dummy;
 };
 
@@ -160,6 +173,7 @@ enum
   PRINTER_PAUSED_COLUMN,
   PRINTER_DEFAULT_ICON_COLUMN,
   PRINTER_ICON_COLUMN,
+  PRINTER_IS_SEPARATOR_COLUMN,
   PRINTER_N_COLUMNS
 };
 
@@ -171,19 +185,21 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
   CcPrintersPanel        *self = (CcPrintersPanel*) user_data;
   GtkTreeModel           *model;
   GtkTreeIter             iter;
-  const gchar            *none = "---";
   GtkWidget              *widget;
   gboolean                test_page_command_available = FALSE;
+  gboolean                sensitive;
   gchar                  *printer_make_and_model = NULL;
   gchar                  *reason = NULL;
   gchar                 **available_commands = NULL;
   gchar                  *printer_commands = NULL;
   gchar                 **printer_reasons = NULL;
-  gchar                  *marker_levels = NULL;
+  gchar                  *marker_types = NULL;
   gchar                  *printer_name = NULL;
   gchar                  *active_jobs = NULL;
+  gchar                  *supply_type = NULL;
   gchar                  *location = NULL;
   gchar                  *status = NULL;
+  guint                   num_jobs;
   int                     printer_state = 3;
   int                     id = -1;
   int                     i, j;
@@ -231,9 +247,9 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
       /* Translators: At least one input tray is empty */
       N_("Out of paper"),
       /* Translators: The printer is offline */
-      N_("Offline"),
+      NC_("printer state", "Offline"),
       /* Translators: Someone has paused the Printer */
-      N_("Paused"),
+      NC_("printer state", "Paused"),
       /* Translators: The printer marker supply waste receptacle is almost full */
       N_("Waste receptacle almost full"),
       /* Translators: The printer marker supply waste receptacle is full */
@@ -273,8 +289,8 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
             printer_state = atoi (priv->dests[id].options[i].value);
           else if (g_strcmp0 (priv->dests[id].options[i].name, "printer-state-reasons") == 0)
             reason = priv->dests[id].options[i].value;
-          else if (g_strcmp0 (priv->dests[priv->current_dest].options[i].name, "marker-levels") == 0)
-            marker_levels = priv->dests[priv->current_dest].options[i].value;
+          else if (g_strcmp0 (priv->dests[priv->current_dest].options[i].name, "marker-types") == 0)
+            marker_types = priv->dests[priv->current_dest].options[i].value;
           else if (g_strcmp0 (priv->dests[priv->current_dest].options[i].name, "printer-commands") == 0)
             printer_commands = priv->dests[priv->current_dest].options[i].value;
           else if (g_strcmp0 (priv->dests[priv->current_dest].options[i].name, "printer-make-and-model") == 0)
@@ -333,15 +349,15 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
             {
               case 3:
                 /* Translators: Printer's state (can start new job without waiting) */
-                status = g_strdup ( _("Idle"));
+                status = g_strdup ( C_("printer state", "Ready"));
                 break;
               case 4:
                 /* Translators: Printer's state (jobs are processing) */
-                status = g_strdup ( _("Processing"));
+                status = g_strdup ( C_("printer state", "Processing"));
                 break;
               case 5:
                 /* Translators: Printer's state (no jobs can be processed) */
-                status = g_strdup ( _("Stopped"));
+                status = g_strdup ( C_("printer state", "Stopped"));
                 break;
             }
         }
@@ -355,7 +371,7 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
           g_free (printer_name);
         }
       else
-        gtk_label_set_text (GTK_LABEL (widget), none);
+        gtk_label_set_text (GTK_LABEL (widget), EMPTY_TEXT);
 
         
       widget = (GtkWidget*)
@@ -367,7 +383,7 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
           g_free (status);
         }
       else
-        gtk_label_set_text (GTK_LABEL (widget), none);
+        gtk_label_set_text (GTK_LABEL (widget), EMPTY_TEXT);
 
 
       widget = (GtkWidget*)
@@ -379,7 +395,7 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
           g_free (location);
         }
       else
-        gtk_label_set_text (GTK_LABEL (widget), none);
+        gtk_label_set_text (GTK_LABEL (widget), EMPTY_TEXT);
 
 
       widget = (GtkWidget*)
@@ -391,13 +407,12 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
           g_free (printer_make_and_model);
         }
       else
-        gtk_label_set_text (GTK_LABEL (widget), none);
+        gtk_label_set_text (GTK_LABEL (widget), EMPTY_TEXT);
 
 
       widget = (GtkWidget*)
         gtk_builder_get_object (priv->builder, "printer-disable-switch");
 
-      gtk_widget_set_sensitive (widget, TRUE);
       g_signal_handlers_block_by_func (G_OBJECT (widget), printer_disable_cb, self);
       gtk_switch_set_active (GTK_SWITCH (widget), printer_state != 5);
       g_signal_handlers_unblock_by_func (G_OBJECT (widget), printer_disable_cb, self);
@@ -406,10 +421,11 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
       widget = (GtkWidget*)
         gtk_builder_get_object (priv->builder, "printer-default-check-button");
 
-      gtk_widget_set_sensitive (widget, TRUE);
+      sensitive = gtk_widget_get_sensitive (widget);
       g_signal_handlers_block_by_func (G_OBJECT (widget), printer_set_default_cb, self);
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), priv->dests[id].is_default);
       g_signal_handlers_unblock_by_func (G_OBJECT (widget), printer_set_default_cb, self);
+      gtk_widget_set_sensitive (widget, sensitive);
 
 
       if (printer_commands)
@@ -430,31 +446,37 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
 
       widget = (GtkWidget*)
         gtk_builder_get_object (priv->builder, "supply-drawing-area");
-
-      if (marker_levels)
-        {
-          gchar **marker_levelsv = NULL;
-
-          widget = (GtkWidget*)
-            gtk_builder_get_object (priv->builder, "supply-drawing-area");
-
-          marker_levelsv = g_strsplit (marker_levels, ",", -1);
-          gtk_widget_set_size_request (widget,
-                                       -1,
-                                       ((g_strv_length (marker_levelsv) - 1) * SUPPLY_BAR_SPACE
-                                       + g_strv_length (marker_levelsv) * SUPPLY_BAR_HEIGHT));
-          g_strfreev (marker_levelsv);
-        }
-      else
-        gtk_widget_set_size_request (widget, -1, -1);
-
+      gtk_widget_set_size_request (widget, -1, SUPPLY_BAR_HEIGHT);
       gtk_widget_queue_draw (widget);
 
 
+      widget = (GtkWidget*)
+        gtk_builder_get_object (priv->builder, "supply-label");
+
+      if (marker_types && g_strrstr (marker_types, "toner") != NULL)
+        /* Translators: Toner supply */
+        supply_type = g_strdup ( _("Toner Level"));
+      else if (marker_types && g_strrstr (marker_types, "ink") != NULL)
+        /* Translators: Ink supply */
+        supply_type = g_strdup ( _("Ink Level"));
+      else
+        /* Translators: By supply we mean ink, toner, staples, water, ... */
+        supply_type = g_strdup ( _("Supply Level"));
+
+      if (supply_type)
+        {
+          gtk_label_set_text (GTK_LABEL (widget), supply_type);
+          g_free (supply_type);
+        }
+      else
+        gtk_label_set_text (GTK_LABEL (widget), EMPTY_TEXT);
+
+
       widget = (GtkWidget*)
         gtk_builder_get_object (priv->builder, "printer-jobs-label");
+      num_jobs = priv->num_jobs < 0 ? 0 : (guint) priv->num_jobs;
       /* Translators: there is n active print jobs on this printer */
-      active_jobs = g_strdup_printf (_("%d active"), priv->num_jobs);
+      active_jobs = g_strdup_printf (ngettext ("%u active", "%u active", num_jobs), num_jobs);
 
       if (active_jobs)
         {
@@ -462,7 +484,16 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
           g_free (active_jobs);
         }
       else
-        gtk_label_set_text (GTK_LABEL (widget), none);
+        gtk_label_set_text (GTK_LABEL (widget), EMPTY_TEXT);
+
+
+      widget = (GtkWidget*)
+        gtk_builder_get_object (priv->builder, "printer-options-button");
+      gtk_widget_set_sensitive (widget, TRUE);
+
+      widget = (GtkWidget*)
+        gtk_builder_get_object (priv->builder, "printer-jobs-button");
+      gtk_widget_set_sensitive (widget, TRUE);
     }
   else
     {
@@ -483,7 +514,19 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
       gtk_label_set_text (GTK_LABEL (widget), "");
 
       widget = (GtkWidget*)
-        gtk_builder_get_object (priv->builder, "printer-disable-switch");
+        gtk_builder_get_object (priv->builder, "printer-ip-address-label");
+      gtk_label_set_text (GTK_LABEL (widget), "");
+
+      widget = (GtkWidget*)
+        gtk_builder_get_object (priv->builder, "print-test-page-button");
+      gtk_widget_set_sensitive (widget, FALSE);
+
+      widget = (GtkWidget*)
+        gtk_builder_get_object (priv->builder, "printer-options-button");
+      gtk_widget_set_sensitive (widget, FALSE);
+
+      widget = (GtkWidget*)
+        gtk_builder_get_object (priv->builder, "printer-jobs-button");
       gtk_widget_set_sensitive (widget, FALSE);
     }
 
@@ -500,6 +543,26 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
   gtk_widget_set_sensitive (widget, FALSE);
 }
 
+typedef struct {
+  gint printer_id;
+  gint usage;
+} PrinterUsage;
+
+static gint
+printer_usage_cmp (gconstpointer a,
+                   gconstpointer b)
+{
+  PrinterUsage *x = (PrinterUsage*) a;
+  PrinterUsage *y = (PrinterUsage*) b;
+
+  if (x->usage < y->usage)
+    return 1;
+  else if (x->usage == y->usage)
+    return 0;
+  else
+    return -1;
+}
+
 static void
 actualize_printers_list (CcPrintersPanel *self)
 {
@@ -508,13 +571,20 @@ actualize_printers_list (CcPrintersPanel *self)
   GtkTreeIter             selected_iter;
   GtkTreeView            *treeview;
   GtkTreeIter             iter;
+  cups_job_t             *jobs = NULL;
+  gboolean                has_used_printer = FALSE;
+  gboolean                has_separator = FALSE;
   gboolean                paused = FALSE;
+  gboolean                valid = FALSE;
   gchar                  *current_printer_instance = NULL;
   gchar                  *current_printer_name = NULL;
   gchar                  *printer_icon_name = NULL;
   gchar                  *default_icon_name = NULL;
+  GList                  *tmp_list = NULL;
+  GList                  *usages = NULL;
   int                     current_dest = -1;
   int                     i, j;
+  int                     num_jobs = 0;
 
   priv = PRINTERS_PANEL_PRIVATE (self);
 
@@ -540,12 +610,47 @@ actualize_printers_list (CcPrintersPanel *self)
                               G_TYPE_STRING,
                               G_TYPE_BOOLEAN,
                               G_TYPE_STRING,
-                              G_TYPE_STRING);
+                              G_TYPE_STRING,
+                              G_TYPE_BOOLEAN);
 
   for (i = 0; i < priv->num_dests; i++)
+    {
+      PrinterUsage *usage = g_new (PrinterUsage, 1);
+      num_jobs = cupsGetJobs (&jobs, priv->dests[i].name, 1, CUPS_WHICHJOBS_ALL);
+      usage->printer_id = i;
+      usage->usage = num_jobs;
+      usages = g_list_append (usages, usage);
+      cupsFreeJobs (num_jobs, jobs);
+    }
+  num_jobs = 0;
+  jobs = NULL;
+
+  usages = g_list_sort (usages, printer_usage_cmp);
+
+  for (tmp_list = usages; tmp_list; tmp_list = tmp_list->next)
     {
       gchar *instance;
 
+      i = ((PrinterUsage*) tmp_list->data)->printer_id;
+      if (((PrinterUsage*) tmp_list->data)->usage == 0)
+        {
+          if (!has_separator && has_used_printer)
+            {
+              has_separator = TRUE;
+              gtk_list_store_append (store, &iter);
+              gtk_list_store_set (store, &iter,
+                                  PRINTER_ID_COLUMN, -1,
+                                  PRINTER_NAME_COLUMN, NULL,
+                                  PRINTER_PAUSED_COLUMN, FALSE,
+                                  PRINTER_DEFAULT_ICON_COLUMN, FALSE,
+                                  PRINTER_ICON_COLUMN, NULL,
+                                  PRINTER_IS_SEPARATOR_COLUMN, TRUE,
+                                  -1);
+            }
+        }
+      else
+        has_used_printer = TRUE;
+
       gtk_list_store_append (store, &iter);
 
       if (priv->dests[i].instance)
@@ -579,7 +684,7 @@ actualize_printers_list (CcPrintersPanel *self)
         }
 
       if (priv->dests[i].is_default)
-        default_icon_name = g_strdup ("gtk-apply");
+        default_icon_name = g_strdup ("emblem-default-symbolic");
       else
         default_icon_name = NULL;
 
@@ -591,12 +696,14 @@ actualize_printers_list (CcPrintersPanel *self)
                           PRINTER_PAUSED_COLUMN, paused,
                           PRINTER_DEFAULT_ICON_COLUMN, default_icon_name,
                           PRINTER_ICON_COLUMN, printer_icon_name,
+                          PRINTER_IS_SEPARATOR_COLUMN, FALSE,
                           -1);
 
       g_free (instance);
       g_free (printer_icon_name);
       g_free (default_icon_name);
     }
+  g_list_free_full (usages, g_free);
 
   gtk_tree_view_set_model (treeview, GTK_TREE_MODEL (store));
 
@@ -609,9 +716,6 @@ actualize_printers_list (CcPrintersPanel *self)
     }
   else
     {
-      cups_job_t *jobs = NULL;
-      int         num_jobs = 0;
-
       num_jobs = cupsGetJobs (&jobs, NULL, 1, CUPS_WHICHJOBS_ALL);
 
       /* Select last used printer */
@@ -637,24 +741,37 @@ actualize_printers_list (CcPrintersPanel *self)
               }
         }
 
-      /* Select first printer */
-      if (priv->current_dest < 0 && priv->num_dests > 0)
-        {
-          priv->current_dest = 0;
-        }
-
       if (priv->current_dest >= 0)
         {
-          GtkTreePath *path = gtk_tree_path_new_from_indices (priv->current_dest, -1);
+          gint id;
+          valid = gtk_tree_model_get_iter_first ((GtkTreeModel *) store,
+                                                 &selected_iter);
 
-          gtk_tree_model_get_iter ((GtkTreeModel *) store,
-                                   &selected_iter,
-                                   path);
+          while (valid)
+            {
+              gtk_tree_model_get ((GtkTreeModel *) store, &selected_iter,
+                                  PRINTER_ID_COLUMN, &id,
+                                  -1);
+              if (id == priv->current_dest)
+                break;
+
+              valid = gtk_tree_model_iter_next ((GtkTreeModel *) store,
+                                                &selected_iter);
+            }
+
+          gtk_tree_selection_select_iter (
+            gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview)),
+            &selected_iter);
+        }
+      else if (priv->num_dests > 0)
+        {
+          /* Select first printer */
+          gtk_tree_model_get_iter_first ((GtkTreeModel *) store,
+                                         &selected_iter);
 
           gtk_tree_selection_select_iter (
             gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview)),
             &selected_iter);
-          gtk_tree_path_free (path);
         }
     }
 
@@ -684,6 +801,23 @@ set_cell_sensitivity_func (GtkTreeViewColumn *tree_column,
                   NULL);
 }
 
+static gboolean
+printers_row_separator_func (GtkTreeModel *model,
+                             GtkTreeIter  *iter,
+                             gpointer      data)
+{
+  gboolean separator;
+
+  gtk_tree_model_get (model,
+                      iter,
+                      PRINTER_IS_SEPARATOR_COLUMN,
+                      &separator,
+                      -1);
+
+  return separator;
+}
+
+
 static void
 populate_printers_list (CcPrintersPanel *self)
 {
@@ -714,19 +848,26 @@ populate_printers_list (CcPrintersPanel *self)
 
 
   renderer = gtk_cell_renderer_text_new ();
+  g_object_set (G_OBJECT (renderer), "ellipsize", PANGO_ELLIPSIZE_END, "width-chars", 18, NULL);
   column = gtk_tree_view_column_new_with_attributes ("Printer", renderer,
                                                      "text", PRINTER_NAME_COLUMN, NULL);
   gtk_tree_view_column_set_cell_data_func (column, renderer, set_cell_sensitivity_func,
                                            self, NULL);
   gtk_tree_view_column_set_expand (column, FALSE);
+  gtk_tree_view_column_set_min_width (column, 120);
   gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 
 
   icon_renderer2 = gtk_cell_renderer_pixbuf_new ();
   column = gtk_tree_view_column_new_with_attributes ("Default", icon_renderer2,
-                                                     "stock-id", PRINTER_DEFAULT_ICON_COLUMN, NULL);
+                                                     "icon-name", PRINTER_DEFAULT_ICON_COLUMN, NULL);
   gtk_tree_view_column_set_expand (column, FALSE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
+
+  gtk_tree_view_set_row_separator_func (GTK_TREE_VIEW (treeview),
+                                        printers_row_separator_func,
+                                        NULL,
+                                        NULL);
 }
 
 enum
@@ -734,7 +875,6 @@ enum
   JOB_ID_COLUMN,
   JOB_TITLE_COLUMN,
   JOB_STATE_COLUMN,
-  JOB_USER_COLUMN,
   JOB_CREATION_TIME_COLUMN,
   JOB_N_COLUMNS
 };
@@ -746,6 +886,7 @@ actualize_jobs_list (CcPrintersPanel *self)
   GtkListStore           *store;
   GtkTreeView            *treeview;
   GtkTreeIter             iter;
+  GSettings              *settings;
   int                     i;
 
   priv = PRINTERS_PANEL_PRIVATE (self);
@@ -764,46 +905,63 @@ actualize_jobs_list (CcPrintersPanel *self)
       priv->dests != NULL)
     priv->num_jobs = cupsGetJobs (&priv->jobs, priv->dests[priv->current_dest].name, 1, CUPS_WHICHJOBS_ACTIVE);
 
-  store = gtk_list_store_new (JOB_N_COLUMNS, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+  store = gtk_list_store_new (JOB_N_COLUMNS, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
 
   for (i = 0; i < priv->num_jobs; i++)
     {
+      GDesktopClockFormat  value;
+      GDateTime           *time;
       struct tm *ts;
       gchar     *time_string;
       gchar     *state = NULL;
 
       ts = localtime (&(priv->jobs[i].creation_time));
-      time_string = g_strdup_printf ("%02d:%02d:%02d", ts->tm_hour, ts->tm_min, ts->tm_sec);
+      time = g_date_time_new_local (ts->tm_year,
+                                    ts->tm_mon,
+                                    ts->tm_mday,
+                                    ts->tm_hour,
+                                    ts->tm_min,
+                                    ts->tm_sec);
+
+      settings = g_settings_new (CLOCK_SCHEMA);
+      value = g_settings_get_enum (settings, CLOCK_FORMAT_KEY);
+
+      if (value == G_DESKTOP_CLOCK_FORMAT_24H)
+        time_string = g_date_time_format (time, "%k:%M");
+      else
+        time_string = g_date_time_format (time, "%l:%M %p");
+
+      g_date_time_unref (time);
 
       switch (priv->jobs[i].state)
         {
           case IPP_JOB_PENDING:
             /* Translators: Job's state (job is waiting to be printed) */
-            state = g_strdup (_("Pending"));
+            state = g_strdup (C_("print job", "Pending"));
             break;
           case IPP_JOB_HELD:
             /* Translators: Job's state (job is held for printing) */
-            state = g_strdup (_("Held"));
+            state = g_strdup (C_("print job", "Held"));
             break;
           case IPP_JOB_PROCESSING:
             /* Translators: Job's state (job is currently printing) */
-            state = g_strdup (_("Processing"));
+            state = g_strdup (C_("print job", "Processing"));
             break;
           case IPP_JOB_STOPPED:
             /* Translators: Job's state (job has been stopped) */
-            state = g_strdup (_("Stopped"));
+            state = g_strdup (C_("print job", "Stopped"));
             break;
           case IPP_JOB_CANCELED:
             /* Translators: Job's state (job has been canceled) */
-            state = g_strdup (_("Canceled"));
+            state = g_strdup (C_("print job", "Canceled"));
             break;
           case IPP_JOB_ABORTED:
             /* Translators: Job's state (job has aborted due to error) */
-            state = g_strdup (_("Aborted"));
+            state = g_strdup (C_("print job", "Aborted"));
             break;
           case IPP_JOB_COMPLETED:
             /* Translators: Job's state (job has completed successfully) */
-            state = g_strdup (_("Completed"));
+            state = g_strdup (C_("print job", "Completed"));
             break;
         }
 
@@ -812,7 +970,6 @@ actualize_jobs_list (CcPrintersPanel *self)
                           JOB_ID_COLUMN, i,
                           JOB_TITLE_COLUMN, priv->jobs[i].title,
                           JOB_STATE_COLUMN, state,
-                          JOB_USER_COLUMN, priv->jobs[i].user,
                           JOB_CREATION_TIME_COLUMN, time_string,
                           -1);
 
@@ -882,9 +1039,8 @@ populate_jobs_list (CcPrintersPanel *self)
   CcPrintersPanelPrivate *priv;
   GtkTreeViewColumn      *column;
   GtkCellRenderer        *renderer;
+  GtkCellRenderer        *title_renderer;
   GtkTreeView            *treeview;
-  gfloat                  x_align = 0.5;
-  gfloat                  y_align = 0.5;
 
   priv = PRINTERS_PANEL_PRIVATE (self);
 
@@ -894,12 +1050,12 @@ populate_jobs_list (CcPrintersPanel *self)
     gtk_builder_get_object (priv->builder, "job-treeview");
 
   renderer = gtk_cell_renderer_text_new ();
-  gtk_cell_renderer_get_alignment (renderer, &x_align, &y_align);
-  gtk_cell_renderer_set_alignment (renderer, 0.5, y_align);
+  title_renderer = gtk_cell_renderer_text_new ();
 
   /* Translators: Name of column showing titles of print jobs */
-  column = gtk_tree_view_column_new_with_attributes (_("Job Title"), renderer,
+  column = gtk_tree_view_column_new_with_attributes (_("Job Title"), title_renderer,
                                                      "text", JOB_TITLE_COLUMN, NULL);
+  g_object_set (G_OBJECT (title_renderer), "ellipsize", PANGO_ELLIPSIZE_END, NULL);
   gtk_tree_view_column_set_fixed_width (column, 180);
   gtk_tree_view_column_set_min_width (column, 180);
   gtk_tree_view_column_set_max_width (column, 180);
@@ -911,12 +1067,6 @@ populate_jobs_list (CcPrintersPanel *self)
   gtk_tree_view_column_set_expand (column, TRUE);
   gtk_tree_view_append_column (treeview, column);
 
-  /* Translators: Name of column showing names of creators of print jobs */
-   column = gtk_tree_view_column_new_with_attributes (_("User"), renderer,
-                                                     "text", JOB_USER_COLUMN, NULL);
-  gtk_tree_view_column_set_expand (column, TRUE);
-  gtk_tree_view_append_column (treeview, column);
-
   /* Translators: Name of column showing times of creation of print jobs */
   column = gtk_tree_view_column_new_with_attributes (_("Time"), renderer,
                                                      "text", JOB_CREATION_TIME_COLUMN, NULL);
@@ -1266,6 +1416,39 @@ printer_disable_cb (GObject    *gobject,
     }
 }
 
+typedef struct {
+  gchar *color;
+  gchar *type;
+  gchar *name;
+  gint   level;
+} MarkerItem;
+
+static gint
+markers_cmp (gconstpointer a,
+             gconstpointer b)
+{
+  MarkerItem *x = (MarkerItem*) a;
+  MarkerItem *y = (MarkerItem*) b;
+
+  if (x->level < y->level)
+    return 1;
+  else if (x->level == y->level)
+    return 0;
+  else
+    return -1;
+}
+
+static void
+rounded_rectangle (cairo_t *cr, double x, double y, double w, double h, double r)
+{
+    cairo_new_sub_path (cr);
+    cairo_arc (cr, x + r, y + r, r, M_PI, 3 * M_PI / 2);
+    cairo_arc (cr, x + w - r, y + r, r, 3 *M_PI / 2, 2 * M_PI);
+    cairo_arc (cr, x + w - r, y + h - r, r, 0, M_PI / 2);
+    cairo_arc (cr, x + r, y + h - r, r, M_PI / 2, M_PI);
+    cairo_close_path (cr);
+}
+
 static gboolean
 supply_levels_draw_cb (GtkWidget *widget,
                        cairo_t *cr,
@@ -1273,9 +1456,11 @@ supply_levels_draw_cb (GtkWidget *widget,
 {
   CcPrintersPanelPrivate *priv;
   CcPrintersPanel        *self = (CcPrintersPanel*) user_data;
+  GtkStyleContext        *context;
   gchar                  *marker_levels = NULL;
   gchar                  *marker_colors = NULL;
   gchar                  *marker_names = NULL;
+  gchar                  *marker_types = NULL;
   gchar                  *tooltip_text = NULL;
   gint                    width;
   gint                    height;
@@ -1283,17 +1468,17 @@ supply_levels_draw_cb (GtkWidget *widget,
 
   priv = PRINTERS_PANEL_PRIVATE (self);
 
+  width = gtk_widget_get_allocated_width (widget);
+  height = gtk_widget_get_allocated_height (widget);
+
+  cairo_rectangle (cr, 0.0, 0.0, width, height);
+  gdk_cairo_set_source_rgba (cr, &priv->background_color);
+  cairo_fill (cr);
+
   if (priv->current_dest >= 0 &&
       priv->current_dest < priv->num_dests &&
       priv->dests != NULL)
     {
-      width = gtk_widget_get_allocated_width (widget);
-      height = gtk_widget_get_allocated_height (widget);
-
-      cairo_rectangle (cr, 0.0, 0.0, width, height);
-      gdk_cairo_set_source_rgba (cr, &priv->background_color);
-      cairo_fill (cr);
-
       for (i = 0; i < priv->dests[priv->current_dest].num_options; i++)
         {
           if (g_strcmp0 (priv->dests[priv->current_dest].options[i].name, "marker-names") == 0)
@@ -1302,14 +1487,29 @@ supply_levels_draw_cb (GtkWidget *widget,
             marker_levels = priv->dests[priv->current_dest].options[i].value;
           else if (g_strcmp0 (priv->dests[priv->current_dest].options[i].name, "marker-colors") == 0)
             marker_colors = priv->dests[priv->current_dest].options[i].value;
+          else if (g_strcmp0 (priv->dests[priv->current_dest].options[i].name, "marker-types") == 0)
+            marker_types = priv->dests[priv->current_dest].options[i].value;
         }
 
-      if (marker_levels && marker_colors && marker_names)
+      if (marker_levels && marker_colors && marker_names && marker_types)
         {
-          gchar **marker_levelsv = NULL;
-          gchar **marker_colorsv = NULL;
-          gchar **marker_namesv = NULL;
-          gchar  *tmp = NULL;
+          GdkRGBA   border_color = {0.0, 0.0, 0.0, 1.0};
+          GSList   *markers = NULL;
+          GSList   *tmp_list = NULL;
+          GValue    int_val = {0};
+          gchar   **marker_levelsv = NULL;
+          gchar   **marker_colorsv = NULL;
+          gchar   **marker_namesv = NULL;
+          gchar   **marker_typesv = NULL;
+          gchar    *tmp = NULL;
+          gint      border_radius = 3;
+
+          context = gtk_widget_get_style_context ((GtkWidget *)
+            gtk_builder_get_object (priv->builder, "printer-options-button"));
+          gtk_style_context_get_border_color (context, 0, &border_color);
+          gtk_style_context_get_property (
+            context, GTK_STYLE_PROPERTY_BORDER_RADIUS, 0, &int_val);
+          border_radius = g_value_get_int (&int_val);
 
           widget = (GtkWidget*)
             gtk_builder_get_object (priv->builder, "supply-drawing-area");
@@ -1317,68 +1517,76 @@ supply_levels_draw_cb (GtkWidget *widget,
           marker_levelsv = g_strsplit (marker_levels, ",", -1);
           marker_colorsv = g_strsplit (marker_colors, ",", -1);
           marker_namesv = g_strsplit (marker_names, ",", -1);
+          marker_typesv = g_strsplit (marker_types, ",", -1);
   
-          width = width < SUPPLY_BAR_WIDTH ? width : SUPPLY_BAR_WIDTH;
           for (i = 0; i < g_strv_length (marker_levelsv); i++)
+            {
+              MarkerItem *marker;
+
+              if (g_strcmp0 (marker_typesv[i], "ink") == 0 ||
+                  g_strcmp0 (marker_typesv[i], "toner") == 0)
+                {
+                  marker = g_new0 (MarkerItem, 1);
+                  marker->type = g_strdup (marker_typesv[i]);
+                  marker->name = g_strdup (marker_namesv[i]);
+                  marker->color = g_strdup (marker_colorsv[i]);
+                  marker->level = atoi (marker_levelsv[i]);
+
+                  markers = g_slist_prepend (markers, marker);
+                }
+            }
+
+          markers = g_slist_sort (markers, markers_cmp);
+
+          for (tmp_list = markers; tmp_list; tmp_list = tmp_list->next)
             {
               GdkRGBA color = {0.0, 0.0, 0.0, 1.0};
-              GdkRGBA light_color;
               double  display_value;
               int     value;
 
-              value = atoi (marker_levelsv[i]);
+              value = ((MarkerItem*) tmp_list->data)->level;
 
-              gdk_rgba_parse (&color, marker_colorsv[i]);
-              light_color.red = color.red < 0.8 ? color.red + 0.8 : 1.0;
-              light_color.green = color.green < 0.8 ? color.green + 0.8 : 1.0;
-              light_color.blue = color.blue < 0.8 ? color.blue + 0.8 : 1.0;
-              light_color.alpha = 1.0;
+              gdk_rgba_parse (&color, ((MarkerItem*) tmp_list->data)->color);
 
-              if (value >= 0)
+              if (value > 0)
                 {
-                  display_value = value / 100.0 * width;
-
-                  cairo_rectangle (cr,
-                                   0.0,
-                                   i * (SUPPLY_BAR_HEIGHT + SUPPLY_BAR_SPACE),
-                                   display_value,
-                                   SUPPLY_BAR_HEIGHT);
+                  display_value = value / 100.0 * (width - 3.0);
                   gdk_cairo_set_source_rgba (cr, &color);
-                  cairo_fill (cr);
-
-                  cairo_rectangle (cr,
-                                   display_value,
-                                   i * (SUPPLY_BAR_HEIGHT + SUPPLY_BAR_SPACE),
-                                   width - display_value,
-                                   SUPPLY_BAR_HEIGHT);
-                  gdk_cairo_set_source_rgba (cr, &light_color);
-                  cairo_fill (cr);
-                }
-              else
-                {
-                  cairo_rectangle (cr,
-                                   0.0,
-                                   i * (SUPPLY_BAR_HEIGHT + SUPPLY_BAR_SPACE),
-                                   width,
-                                   SUPPLY_BAR_HEIGHT);
-                  gdk_cairo_set_source_rgba (cr, &light_color);
+                  rounded_rectangle (cr, 1.5, 1.5, display_value, SUPPLY_BAR_HEIGHT - 3.0, border_radius);
                   cairo_fill (cr);
                 }
 
               if (tooltip_text)
                 {
-                  tmp = g_strdup_printf ("%s\n%s", tooltip_text, marker_namesv[i]);
+                  tmp = g_strdup_printf ("%s\n%s",
+                                         tooltip_text,
+                                         ((MarkerItem*) tmp_list->data)->name);
                   g_free (tooltip_text);
                   tooltip_text = tmp;
                   tmp = NULL;
                 }
               else
-                tooltip_text = g_strdup_printf ("%s", marker_namesv[i]);
+                tooltip_text = g_strdup_printf ("%s",
+                                                ((MarkerItem*) tmp_list->data)->name);
             }
 
+          cairo_set_line_width (cr, 1.0);
+          gdk_cairo_set_source_rgba (cr, &border_color);
+          rounded_rectangle (cr, 1.5, 1.5, width - 3.0, SUPPLY_BAR_HEIGHT - 3.0, border_radius);
+          cairo_stroke (cr);
+
+          for (tmp_list = markers; tmp_list; tmp_list = tmp_list->next)
+            {
+              g_free (((MarkerItem*) tmp_list->data)->name);
+              g_free (((MarkerItem*) tmp_list->data)->type);
+              g_free (((MarkerItem*) tmp_list->data)->color);
+            }
+          g_slist_free_full (markers, g_free);
+
           g_strfreev (marker_levelsv);
           g_strfreev (marker_colorsv);
           g_strfreev (marker_namesv);
+          g_strfreev (marker_typesv);
         }
 
       g_free (marker_names);
@@ -1779,6 +1987,39 @@ printer_maintenance_cb (GtkButton *button,
     }
 }
 
+static void
+on_permission_changed (GPermission *permission,
+                       GParamSpec  *pspec,
+                       gpointer     data)
+{
+  CcPrintersPanelPrivate  *priv;
+  CcPrintersPanel         *self = (CcPrintersPanel*) data;
+  GtkWidget               *widget;
+  gboolean                 is_authorized;
+
+  priv = PRINTERS_PANEL_PRIVATE (self);
+
+  is_authorized = g_permission_get_allowed (G_PERMISSION (priv->permission));
+
+  widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "printer-add-button");
+  gtk_widget_set_sensitive (widget, is_authorized);
+
+  widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "printer-remove-button");
+  gtk_widget_set_sensitive (widget, is_authorized);
+
+  widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "printer-disable-switch");
+  gtk_widget_set_sensitive (widget, is_authorized);
+
+  widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "printer-default-check-button");
+  gtk_widget_set_sensitive (widget, is_authorized);
+
+  widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "allowed-user-add-button");
+  gtk_widget_set_sensitive (widget, is_authorized);
+
+  widget = (GtkWidget*) gtk_builder_get_object (priv->builder, "allowed-user-remove-button");
+  gtk_widget_set_sensitive (widget, is_authorized);
+}
+
 enum
 {
   NOTEBOOK_INFO_PAGE = 0,
@@ -1838,6 +2079,7 @@ cc_printers_panel_init (CcPrintersPanel *self)
   CcPrintersPanelPrivate *priv;
   GtkWidget              *top_widget;
   GtkWidget              *widget;
+  GtkWidget              *box;
   GError                 *error = NULL;
   gchar                  *objects[] = { "main-vbox", NULL };
   GtkStyleContext        *context;
@@ -1923,11 +2165,11 @@ cc_printers_panel_init (CcPrintersPanel *self)
     gtk_builder_get_object (priv->builder, "back-button-2");
   g_signal_connect (widget, "clicked", G_CALLBACK (go_back_cb), self);
 
-   widget = (GtkWidget*)
+  widget = (GtkWidget*)
     gtk_builder_get_object (priv->builder, "printer-jobs-button");
   g_signal_connect (widget, "clicked", G_CALLBACK (switch_to_jobs_cb), self);
 
-   widget = (GtkWidget*)
+  widget = (GtkWidget*)
     gtk_builder_get_object (priv->builder, "printer-options-button");
   g_signal_connect (widget, "clicked", G_CALLBACK (switch_to_options_cb), self);
 
@@ -1963,6 +2205,24 @@ cc_printers_panel_init (CcPrintersPanel *self)
   context = gtk_widget_get_style_context (widget);
   gtk_style_context_set_junction_sides (context, GTK_JUNCTION_TOP);
 
+
+  /* Add unlock button */
+  priv->permission = (GPermission *)polkit_permission_new_sync (
+    "org.opensuse.cupspkhelper.mechanism.printeraddremove", NULL, NULL, NULL);
+  if (priv->permission != NULL)
+    {
+      widget = pp_lock_button_new (priv->permission);
+      gtk_widget_set_margin_top (widget, 12);
+      gtk_widget_show (widget);
+      box = (GtkWidget*) gtk_builder_get_object (priv->builder, "main-vbox");
+      gtk_box_pack_end (GTK_BOX (box), widget, FALSE, FALSE, 0);
+      g_signal_connect (priv->permission, "notify",
+                        G_CALLBACK (on_permission_changed), self);
+      on_permission_changed (priv->permission, NULL, self);
+      priv->lock_button = widget;
+    }
+
+
   gtk_style_context_get_background_color (gtk_widget_get_style_context (top_widget),
                                           GTK_STATE_FLAG_NORMAL,
                                           &priv->background_color);
