@@ -522,14 +522,11 @@ popup_button_draw (GtkWidget      *widget,
                 return;
         }
 
-        gtk_paint_expander (gtk_widget_get_style (widget),
-                            cr,
-                            gtk_widget_get_state (widget),
-                            widget,
-                            NULL,
-                            gtk_widget_get_allocated_width (widget) - 12,
-                            gtk_widget_get_allocated_height (widget) - 12,
-                            GTK_EXPANDER_EXPANDED);
+        down_arrow (gtk_widget_get_style_context (widget),
+                    cr,
+                    gtk_widget_get_allocated_width (widget) - 12,
+                    gtk_widget_get_allocated_height (widget) - 12,
+                    12, 12);
 }
 
 static void
