@@ -154,7 +154,6 @@ activate_panel (GnomeControlCenter *shell,
           gtk_widget_set_size_request (priv->window, FIXED_WIDTH, -1);
           gtk_widget_get_preferred_height (GTK_WIDGET (priv->window),
                                            NULL, &nat_height);
-          g_debug ("Setting panel height: %d", nat_height);
           gtk_window_resize (GTK_WINDOW (priv->window),
                              FIXED_WIDTH,
                              nat_height);
@@ -445,7 +444,7 @@ search_entry_changed_cb (GtkEntry                  *entry,
     {
       shell_show_overview_page (priv);
       g_object_set (G_OBJECT (entry),
-                    "secondary-icon-name", "gtk-find",
+                    "secondary-icon-name", "edit-find-symbolic",
                     "secondary-icon-activatable", FALSE,
                     "secondary-icon-sensitive", FALSE,
                     NULL);
@@ -456,7 +455,7 @@ search_entry_changed_cb (GtkEntry                  *entry,
       gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook),
                                      SEARCH_PAGE);
       g_object_set (G_OBJECT (entry),
-                          "secondary-icon-name", "gtk-clear",
+                          "secondary-icon-name", "edit-clear-symbolic",
                           "secondary-icon-activatable", TRUE,
                           "secondary-icon-sensitive", TRUE,
                           NULL);
@@ -681,15 +680,16 @@ notebook_switch_page_cb (GtkNotebook               *book,
 {
   /* make sure the home button is shown on all pages except the overview page */
 
-  if (page_num == OVERVIEW_PAGE)
-    gtk_widget_hide (W (priv->builder, "home-button"));
+  if (page_num == OVERVIEW_PAGE || page_num == SEARCH_PAGE)
+    {
+      gtk_widget_hide (W (priv->builder, "home-button"));
+      gtk_widget_show (W (priv->builder, "search-entry"));
+    }
   else
-    gtk_widget_show (W (priv->builder, "home-button"));
-
-  if (page_num == CAPPLET_PAGE)
-    gtk_widget_hide (W (priv->builder, "search-entry"));
-  else
-    gtk_widget_show (W (priv->builder, "search-entry"));
+    {
+      gtk_widget_show (W (priv->builder, "home-button"));
+      gtk_widget_hide (W (priv->builder, "search-entry"));
+    }
 }
 
 /* CcShell implementation */
@@ -917,7 +917,6 @@ on_window_size_allocate (GtkWidget          *widget,
 
   if (gtk_scrolled_window_get_min_content_height (GTK_SCROLLED_WINDOW (self->priv->scrolled_window)) != height)
     {
-      g_debug ("Setting min content height: %d", height);
       gtk_scrolled_window_set_min_content_height (GTK_SCROLLED_WINDOW (self->priv->scrolled_window), height);
       /*
        * Queueing a resize out of size-allocate is ignored,
@@ -946,6 +945,35 @@ viewport_style_set_cb (GtkWidget *widget,
   g_signal_handlers_unblock_by_func (widget, viewport_style_set_cb, NULL);
 }
 
+static gboolean
+window_key_press_event (GtkWidget          *win,
+			GdkEventKey        *event,
+			GnomeControlCenter *self)
+{
+  gboolean retval;
+
+  retval = FALSE;
+
+  if (!gtk_widget_get_visible (self->priv->search_entry))
+    return retval;
+
+  if (event->state != 0 &&
+      (event->state & GDK_CONTROL_MASK))
+    {
+      switch (event->keyval)
+        {
+          case GDK_KEY_s:
+          case GDK_KEY_S:
+          case GDK_KEY_f:
+          case GDK_KEY_F:
+            gtk_widget_grab_focus (self->priv->search_entry);
+            retval = TRUE;
+            break;
+        }
+    }
+  return retval;
+}
+
 static void
 gnome_control_center_init (GnomeControlCenter *self)
 {
@@ -969,6 +997,8 @@ gnome_control_center_init (GnomeControlCenter *self)
   /* connect various signals */
   priv->window = W (priv->builder, "main-window");
   g_signal_connect_swapped (priv->window, "delete-event", G_CALLBACK (g_object_unref), self);
+  g_signal_connect (priv->window, "key_press_event",
+                    G_CALLBACK (window_key_press_event), self);
 
   priv->notebook = W (priv->builder, "notebook");
   priv->scrolled_window = W (priv->builder, "scrolledwindow1");
